﻿// Learn more about F# at http://fsharp.net. See the 'F# Tutorial' project
// for more guidance on F# programming.

(*
The code below can be executed in F# Interactive.
For ducumentation refer to https://msdn.microsoft.com/en-us/visualfsharpdocs/conceptual/fsharp-interactive-[fsi.exe]-reference
*)

//Point to the directory that contains the Release bild of the library SpectralEstimation.dll
#r @"C:\Users\chorowski\Google Drive\Visual Studio\Libraries\SpectralEstimation.dll"

open SpectralEstimation
open SampleGenerator
open UtilityFunctions
open Exception
open Estimator

//Contains information necessary to run a single simulation
type ScenarioData =
    {
    Delta :float
    T     :float
    BasisDimension :int
    Diffusion      :DiffusionData
    OccupationDensityLowerBound :float
    }

//Example of a Monte Carlo data
let data=
    {
       Diffusion = { Drift      = fun x -> 0.2 - 0.4 * x;
                     Volatility = fun x -> 0.4 - (x - 0.5) * (x - 0.5) }
       Delta = 0.0001
       T = 5.
       BasisDimension = 25
       OccupationDensityLowerBound = 0.2
    }

//Executes a single simulation (from sample generation to estimation error)
let runScenario (data :ScenarioData) (nr :int) =
    if nr % 100 = 0 then printfn <| Printf.TextWriterFormat<unit>("Processing scenario " + nr.ToString())
    let sample = generateRegularSampleWithMinOccDensity data.Diffusion data.Delta data.T data.OccupationDensityLowerBound
    let basis = new FourierCosineBasis (data.BasisDimension) :> IBasis
    let estimator = new GHRVolatilityEstimator (sample, basis)
    estimator.Error 1. 0.1 0.9 data.Diffusion.Volatility

//Executes specified number of simulations and aggregates the results
let runMonteCarlo (data :ScenarioData) (runs :int) =
    Array.init runs (fun i -> runScenario data i) |> Array.average