﻿// Learn more about F# at http://fsharp.net. See the 'F# Tutorial' project
// for more guidance on F# programming.

(*
The code below can be executed in F# Interactive.
For ducumentation refer to https://msdn.microsoft.com/en-us/visualfsharpdocs/conceptual/fsharp-interactive-[fsi.exe]-reference
*)


//Point to the directory that contains the Release bild of the library SpectralEstimation.dll
#r @"C:\Users\chorowski\Google Drive\Visual Studio\Libraries\SpectralEstimation.dll"
open SpectralEstimation
open SampleGenerator

//Diffusion data
let diffusionData = 
    { Drift      = fun x -> 0.2 - 0.4 * x;
      Volatility = fun x -> 0.4 - (x - 0.5) * (x - 0.5) }

//Basis
let FourierBasis = new LinearSplinesBasis 19 :> IBasis

//Sample
let regularSample = generateRegularSample diffusionData 0.0001 50000
Plot.Sample regularSample
let exponentialSample =
    let distribution = RandomScheduleType.Exponential 0.25
    let schedule = generateRandomSchedule distribution 12000
    generateSampleFromTimeSchedule diffusionData schedule

(*  GOBET, HOFFMANN, REIß VOLATILITY ESTIMATOR *)
let GHREstimator = new GHRVolatilityEstimator (regularSample, FourierBasis)
Plot.Compare GHREstimator.Volatility diffusionData.Volatility     
    
(* FREQUENCY UNIVERSAL VOLATILITY ESTIMATOR *)
let ChEstimator = new ChVolatilityEstimator (regularSample, 19)
ChEstimator.Error 1. 0.1 0.9 diffusionData.Volatility   
Plot.Compare ChEstimator.Volatility diffusionData.Volatility       
         
(*  VOLATILITY ESTIMATION WITH RANDOM OBSERVATION TIMES *)
let estimator = new ChTVolatilityEstimator (exponentialSample, FourierBasis)
estimator.Error 1. 0.1 0.9 diffusionData.Volatility
Plot.Compare estimator.Volatility diffusionData.Volatility